
/* Copyright(C) (2002) (ID - IMAG) <Florent.Morata@imag.fr> */

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

/*! \file analyze.h
 * \brief Header of Analyze class.
 * \author - Florent.Morata@imag.fr
           - Corine.Marchand@imag.fr
	       - Jean-Marc.Vincent@imag.fr
 * \date October 2002.		  
 */

#ifndef _ANALYZE_H_
#define _ANALYZE_H_


#include <vector>
#include <string>
#include <fstream>
#include <iostream>
#include <cmath>

/*! \namespace std */
using namespace std;


/*! Type long double */
typedef long double ldouble;


/*! \class Analyze
 * \brief Statistics treatment of output PSI file '.sample'. 
 */
class Analyze{
  
 private:
  /*! Storage of output generate value. */ 
  vector<int> cl;
  /*! Storage of iteration number values : these values correspond to the coalesce time of an iteration. */
  vector<int> t;
  /*! Sample size. */
  int size;
  /*! Count of different generate values apparition.  */
  vector<int> histo_cl;
  /*! Count of different iteration number values apparition. */
  vector<int> histo_t;
  
 public:
  
  void open_sample(const string path);
  int elt_cl(const int i) const;
  int elt_time(const int i) const;
  int nb_cl() const;
  int size_sample() const;
  int MaxTau() const;
  void create_cl();
  void create_time();
  int elt_histo_time(const int i) const;
  string norm_cl(const string file);
  string norm_time(const string file);
  double pi(const int i) const;
  template<class T> 
    T sum(vector<T> V,const int choice);
  double tau(vector<int> V);
  double sigma(vector<int> V);
  double error(vector<int> V);
  ldouble int_conf(vector<int> V,const int choice);   
};


#include "analyze.inl"



#endif



