
/* Copyright(C) (2002) (ID - IMAG) <Florent.Morata@imag.fr> */

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

/*! \file interface.cpp
 * \brief PSI User Interface.
 * \author - Florent.Morata@imag.fr
           - Corine.Marchand@imag.fr
	   - Jean-Marc.Vincent@imag.fr
 * \date  November 27, 2002.
 */


#ifndef _INTERFACE_CPP_
#define _INTERFACE_CPP_


#include "interface.h"


//---------------------------------------------------------------------------------------------------------------------------


/*! \fn void See_model()
 * \brief Give HBF data in memory.
 */
void See_model()
{
  cout << " " << endl;
  cout << " \n\n    #### MODEL INFORMATIONS #### "<< endl ; 
	
  if(HBF::hbf.is_empty()) cout << "\n No HBF data in memory !" << endl;
  else
    {
      cout << "\n --------------------------------------------------------" << endl;
      cout << "\n Model '" << HBF::hbf.name() << "' in memory" << endl;
      cout << "\n --------------------------------------------------------" << endl;
      cout << "\n - Space states : " << HBF::hbf.order() << endl;
      cout << "\n - Transitions number : " << HBF::hbf.size() << endl;	
    }	
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn void OpenModel(const string path_model)
 * \brief Opening protocol of input models (discrete and continuous).
 */
void OpenModel(const string path_model)
{
  // extension input file. With g++ 2.96 it is possible to write :
  //  string extension = give_ext(path_model);
  string extension = path_model.substr(path_model.find('.'), path_model.length() - path_model.find('.') );	
  if(extension == ".hbf_r")
    {
      HBF::hbf.read_hbf_r(path_model);
      HBF::hbf.update();
    }
  
  if(extension == ".hbf_c")
    {
      HBF::hbf.read_hbf_c(path_model);
      HBF::hbf.col2row();
    }
  
  if(extension == ".marca") HBF::hbf.read_marca(path_model);		
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn void See_simu()
 * \brief Probability estimated results.
 */
void See_simu()
{
  Analyze a;
  string path_sample;
  bool continu = false;
	
  cout << " " << endl;	
  cout << " \n\n    #### SIMULATION RESULTS #### "<< endl ;
  cout << " " << endl;

  while(! continu)
  {
   cout << "   - Input 'sample' file (full path) : ";	
   cin >> path_sample;
   continu = ExistsFile(path_sample);
  }
	  
  a.open_sample(path_sample);
  a.create_cl();
  cout << " " << endl;
  cout << "-------------------------------" << endl;
  cout << "(";	
  for(int i = 0; i < a.nb_cl(); i++)
    cout <<  a.pi(i) << " , " ;
  cout << ")" << endl;
  cout << "-------------------------------" << endl;
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn void Bye()
 * \brief PSI goodbye message. 
 */
void Bye()
{
  cout << " \n\n    ;-) Thanks for using P.S.I. ! " << endl;
  cout << " " << endl;
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn void Credits()
 * \brief PSI credits message.
 */
void Credits()
{
  cout << "\n\n ---------------------------------------------- ";	
  cout << "\n\n   P.S.I. - Perfect Simulation - version 1.0 ";
  cout << "\n\n ---------------------------------------------- ";
  cout << "\n\n PSI is a simulation software for markovian models in discrete and continuous time :";
  cout << "\n\n Simulation kernel is based on the CFTP algorithm of Propp & Wilson and ";
  cout << "\n transitions generation is based on the Aliasing method of Walker.";
  cout << "\n  PSI permit to generate exact sampling of stationary distribution or directly ";
  cout << "\n  functional of stationary distribution.";
  cout << "\n\n  PSI is a free sofware under GPL License and that's why you can redistribute it and/or";
  cout << "\n modify it under the terms of the GNU General Public License as published by the Free Software Foundation."; 
  cout << "\n                          ( see : http://www.fsf.org ) ";
  cout << " " << endl;
  cout << " " << endl;
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn psi_options Menu()
 * \brief PSI interface menu.
 * \return an psi_options corresponding to the asked PSI action.
 */
psi_options Menu()
{
  int option;
  psi_options answer = nothing;
  
  do
    {
      cout << " " << endl;
      cout << "      +-----------------------------------------------+" << endl;
      cout << "      |                                               |" << endl;
      cout << "      |            PSI - Perfect Simulation           |" << endl;
      cout << "      |                                               |" << endl;
      cout << "      |                   ID - IMAG                   |" << endl;
      cout << "      |                                               |" << endl;
      cout << "      +-----------------------------------------------+" << endl;
      cout << "\n (1) Precomputation    (3) Graphical analyze     (5) About this version";
      cout << "\n (2) Simulation        (4) Tools                 (0) Exit ";
      
      cin >> option;
      
      switch(option)
	{
	case 0 : answer = bye;
	  break;
	case 1 : answer = pre;
	  break;
	case 2 : answer = sim;
	  break;
	case 3 : answer = analy;
	  break;
	case 4 : answer = tools;
	  break ;		
	case 5 : answer = credits;
	  break ;
	}
      
    }while(answer == nothing);
  
  return(answer);	
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn void Conversion()
 * \brief Conversion from HBF 'column' format to HBF 'row' format.
 * \return Corresponding file of new (classical) HBF format. 
 */
void Conversion()
{
  bool continu = false;
  string path;
 
  cout << " \n\n    #### HBF CONVERSION #### "<< endl ; 
  cout << " " << endl;
  while(! continu)
    {
      cout << "     :-) HBF 'column' model (full path) : ";
      cin >> path;
      continu = ExistsFile(path);
    }
  
  HBF::hbf.read_hbf_c(path);
  HBF::hbf.convert();
  
  cout << "\n File '" << path.substr(0,path.find('.')) << ".hbf_r' created ...\n";
  
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn void Stat()
 * \brief Statistics results of coalesce time.
 *
 * We give an input 'sample' file to obtain classical statistics results of CFTP algorithm iterations.
 */
void Stat()
{
  Analyze a;
  string path_sample;	
  vector<int> Tmp;
  bool continu = false;
   
  cout << " " << endl;
  cout << " \n\n    #### STATISTICS #### "<< endl ; 
  
 while(! continu)
 {
  cout << " \n\n  - Input 'sample' file : " ;
  cin >> path_sample;
  continu = ExistsFile(path_sample);
 }
 
  a.open_sample(path_sample);
  
  for(int i = 0; i < a.size_sample(); i++) // temporary storage of coalesce time values
    Tmp.push_back(a.elt_time(i));
  cout << "--------------------------------------------------------------------------" << endl;
  cout << " " << endl;
  cout << "  :-) sample size : " << a.size_sample()  << endl; 
  cout << "  :-) coalesce time mean estimated : " <<  a.tau(Tmp) << endl; 
  cout << "  :-) standard deviation estimated : " << setw(3) << a.sigma(Tmp) << endl ;
  cout << "  :-) relative error : " << a.error(Tmp) << endl; 
  cout << "  :-) confidence interval : [" << setw(3) << a.int_conf(Tmp,1) << ";" << a.int_conf(Tmp,2) << "]" << endl; 
  cout << " " << endl;
  cout << "--------------------------------------------------------------------------" << endl;
  
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn void Unif()
 * \brief Uniformization of continuous time model.
 *
 * It returns a 'unif' file which contains discrete equivalent model (in terms of stationary distribution).
 */
void Unif()
{
  string path_model;
  Alias A;
  bool continu = false;
  
  cout << " \n\n    #### UNIFORMIZATION #### "<< endl ; 
  cout << " " << endl;
  while(! continu)
  {
   cout << " - Continuous model (full path) : " ;
   cin >> path_model;
   continu = ExistsFile(path_model);
  }
   
  OpenModel(path_model); // input protocol
  
  HBF::hbf.uniformization(); // uniformization
  HBF::hbf.write_unif(path_model.substr(0,path_model.find('.'))); // 'unif' writing
  
  cout << "\n File '" << path_model.substr(0,path_model.find('.')) << ".unif' created ...\n";
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn void Walk()
 * \brief Aliasing representation for discrete distributions.
 *
 * User has choice to 'row' or 'column' storage of Walker arrays.
 * But be careful !!! Only 'column' storage is adapted to PSI simulation.
 */
void Walk()
{
  string path_model;
  Alias A;
  int D;
  char answ;
  bool continu = false;
  psi_type type = continuous;
	
  cout << "\n\n  #### ALIASING CODING ####" << endl;	

  while(! continu)
  {
	cout << "\n  - Discrete model (full path)  : " ;
    cin >> path_model;
	continu = ExistsFile(path_model);
  }

  cout << " " << endl;
  OpenModel(path_model);
  
  cout << "  - Choice of D (>= " << HBF::hbf.out_degree() << ") : "; // out degree of the transition matrix
  cin >> D; 
  cout << " " << endl;
  
  HBF::hbf.transform(D); // format dimension*D
  
  A.init(D); // first step of aliasing construction		
  A.mk_walker(D); // Walker's construction
  
  cout << "  - Do you want 'row' or 'column' arrays ? (r/c) ";
  cin >> answ;
  
  if(answ == 'r' || answ == 'R') A.write_construct(path_model.substr(0,path_model.find('.')),D);  
  else 
    {
      A.transpose(D);
      A.write_construct(path_model.substr(0,path_model.find('.')),D);  
    }
	
  cout << "\n\n  :-) File '" << path_model.substr(0,path_model.find('.')) << ".simu' created ...\n";
	
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn void Sorry()
 * \brief Temporary unvailable option.
 */
void Sorry()
{
  cout << "\n\n Sorry but this option is not yet operational ! ";	
  cout << " " << endl;
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn bool ExistsFile(const string file_name)
 * \brief Test if a file exists or not.
 * \return true if the file exists, false otherwise.
 */
bool ExistsFile(const string file_name)
{
	FILE *tmp;
	bool value = false;
	tmp = fopen(file_name.c_str(),"rb");
	if(tmp != NULL) value = true;
	if(value) fclose(tmp);
		
	return (value);	
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn bool Overwrite(string file_name)
 * \brief Permisson of overwrite an existing path.
 * \param file_name a string corresponding to the output test file.
 * \return True if user want to overwrite an existing file, false otherwise.
 */
bool Overwrite(string file_name)
{
  char choice;
  bool answer = true;
	
  if(ExistsFile(file_name))
   {
	 cout << "   :-) File '" << file_name << "' already exists ! Do you want to overwrite it ? (y/n)";
	 cin >> choice;
	 cout << " " << endl;
	 answer = ((choice == 'y') || (choice == 'Y'));  
   }
   
	return (answer);
}

//---------------------------------------------------------------------------------------------------------------------------


/*! \fn inline string::iterator give_ext(const string path)
 * \brief Give the extension in a path.
 * \return A string corresponding to the expression '.ext'.
 * \warning Only operational with g++ version 2.96, there is a problem with g++ versio 3.2 to make :
 * - string s = give_ext(path) in an other function.
 */

//---------------------------------------------------------------------------------------------------------------------------

#endif



