
/* Copyright(C) (2002) (ID - IMAG) <Florent.Morata@imag.fr> 

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

/*! \file percent.cpp
 * \brief Methods of Percent class.
 * \author - Florent.Morata@imag.fr
 *         - Corine.Marchand@imag.fr
 *         - Jean-Marc.Vincent@imag.fr
 * \date December 10, 2002.
 */

#ifndef _PERCENT_CPP_
#define _PERCENT_CPP_


// DOXYGEN comments

/*! \fn Percent::Percent()
 * \brief Constructor.
 */

/*! \fn Percent::~Percent()
 * \brief Destructor.
 */

/*! \fn void Percent::progress()
 * \brief Write the percent progression.
 */

/*! \fn inline void Percent::create(const int total,const int step)
 * \brief Calculate the new percent value at each iteration.
 * \param total an integer corresponding to the total number of data.
 * \param step an integer corresponding to the current value of iteration.
 */

/*! \fn inline void Percent::update()
 * \brief Update of percent values.
 */

/*! \fn inline bool Percent::is_new()
 * \brief Return true if new value if different of old value, false otherwise.
 */

/*! \fn inline percent Percent::give_old()
 * \brief Give the old percent value.
 */

/*! \fn inline percent Percent::give_new()
 * \brief Give the new percent value.
 */


#include "percent.h"


//---------------------------------------------------------------------------

Percent::Percent()
{
  old = -1;
  nw = 0;
}

//---------------------------------------------------------------------------

Percent::~Percent()
{
  old = nw = 0;
}

//---------------------------------------------------------------------------

void Percent::progress()
{
  if(is_new()) std::cerr << nw << "%";
}

// ---------------------------------------------------------------------------

#endif



